/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************

File        : RTOSInit_EP7312.c
Purpose     : Initializes and handles the hardware for embOS as far
              as required by embOS for the EP7312 Development Kit.
              Program image starts at 0xc0100000 for that board.
              Feel free to modify this file acc. to your
              target system.
---------------------------END-OF-HEADER------------------------------
*/

/*
  Please note:
  We do automatic release tests for embOS. These tests are done with the
  CPU_KS32C50100. You will also find drivers for additional CPUs, which are
  used by our customers. These drivers have also been tested at least one time,
  but they are not part of our periodic automatic release tests. Therefore it
  might happen, that you encounter a problem with these additional drivers. 
*/

#include "RTOS.H"

void OS_USER_irq_func(void);

/*
**********************************************************************
*
*                    Configuration
*
**********************************************************************
*/

/* Select UART for embOSView, set baudrate,
   define clock frequency for CPU used

   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define OS_UART to -1
*/

#ifndef CPU_EP7312
  #error " This file is for EP7312 CPU only"
#endif

#include "ep7312.h"
#ifndef OS_UART
  #define OS_UART 1
#endif
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif
#define USE_CACHE       0                         /* 0: No chache */
#define OS_TIMER_FREQ   512000                    /* Timer runs at 512 kHz */
#define TIMER_RELOAD    OS_TIMER_FREQ/1000
#define TIMER_PRESCALE  1
#define OS_FSYS         74000000                  /* CPU runs at 74 MHz (check: really ?) */


/*********************************************************************
*
*         Non HW dependent configuration defaults
*
**********************************************************************
*/

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#ifndef OS_FSYS
  #define OS_FSYS 50000000
#endif

#ifndef OS_UART
  #define OS_UART 0
#endif    

#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

#ifndef OS_RAMVECT
  #define OS_RAMVECT 0
#endif


/*
**********************************************************************
*
*                    OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {

  /* Setup MMU and exception table */
  extern void EP7312_MMU_setup(void);
  extern char EP7312_Vectors;
  OS_DI();
  EP7312_MMU_setup();                /* Map RAM at 0x0 */
  memcpy(0, &EP7312_Vectors, 64);    /* Copy exception vector table to 0x0 */
  
  /* Disable all IRQs */
  __INTMR1  = 0x0;          
  __INTMR2  = 0x0;
    
  __SYSCON1 &= ~0x1000;     /* disable LCD */    
  __SYSCON3 = 0x6;          /* Set CPU to 74 MHz  (memory clock: 36 MHz)*/
  __PLLW = 0x28000000;
  __asm { NOP }

  /* Init Timer to generate Tick interrupt  */
  __INTMR1  &= ~0x100ul;     /* disable TC1 underflow interrupt */
  __SYSCON1 |= 0x20ul;       /* Set clock in 512kHz mode */
  __SYSCON1 &= ~ 0x10ul;     /* free running */
  __TC1D    = TIMER_RELOAD;  /* Load counter */
  __INTMR1 |= 0x100ul;       /* enable TC1 underflow interrupt */
  OS_EI();
  OS_COM_Init();
}


/*
**********************************************************************
*
*                     Idle task  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle task.
 This core loop can be changed, but:
 The idle task does not have a stack of its own, therefor no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     // Idle task: No task is ready to exec
  while (1) {
  }
}

/*
**************************************************************
*
*             Run-time error reaction  (OS_Error)
*
**************************************************************

 Run-time error reaction

 When this happens, a fatal error has occured and the kernel
 can not continue. In linux, the equivalent would be a
 "kernel panic"

 This routine can be modified to suit your needs ...
 E.g. a red LED could light up. When using an
 emulator, you may set a breakpoint here.
 In the release builds of the library (R), this routine is not required
 (as no checks are performed).
 In the stack check builds (S/SP), only error 120 may occur.
 In the debug builds(D/DP), all of the listed errors may occur.

  The following are the current errorcodes (V2.80) :
  (Please refer to the documentation for more info)
  
OS_ERR_STACK                          (120)
*** invalid or non-initialized data structures ***
OS_ERR_INV_TASK                       (128)
OS_ERR_INV_TIMER                      (129)
OS_ERR_INV_MAILBOX                    (130)
OS_ERR_INV_CSEMA                      (132)
OS_ERR_INV_RSEMA                      (133)

*** Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on a non-1 byte mailbox  ***
OS_ERR_MAILBOX_NOT1                   (135)

***internal errors, please contact SEGGER Microcontrollersysteme ***
OS_ERR_MAILBOX_NOT_IN_LIST            (140)
OS_ERR_TASKLIST_CORRUPT               (142)


*** not matching routine calls or macro usage ***
OS_ERR_UNUSE_BEFORE_USE               (150)
OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151)
OS_ERR_LEAVEINT                       (152)
OS_ERR_DICNT                          (153)
OS_ERR_INTERRUPT_DISABLED             (154)

*** not a legal system call during interrupt ***
OS_ERR_ILLEGAL_IN_ISR                 (160)
*** not a legal system call during timer ***
OS_ERR_ILLEGAL_IN_TIMER               (161)

** double used data structures **
OS_ERR_2USE_TASK                      (170)
OS_ERR_2USE_TIMER                     (171)
OS_ERR_2USE_MAILBOX                   (172)
OS_ERR_2USE_BSEMA                     (173)
OS_ERR_2USE_CSEMA                     (174)
OS_ERR_2USE_RSEMA                     (175)

*/

void OS_Error(int ErrCode) {
  OS_EnterRegion();     /* Avoid further task switches */
  OS_DICnt =0;          /* Allow interrupts so we can communicate */
  OS_EI();
  OS_Status = ErrCode;
  while (OS_Status);
}


/*
**********************************************************************
*
*                    Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in clock cycles.
*/

#define HW_TIMER  (__TC1D)
#define HW_TIMER_INTERRUPT_PENDING() (__INTSR1 & 0x0100)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TIMER_RELOAD - HW_TIMER;
  OS_U32 time = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = (OS_U32)((OS_I16)((OS_I16)0-(OS_I16)HW_TIMER));
    time++;
  }
  return (OS_TIMER_FREQ/TIMER_PRESCALE/OS_TICK_FREQ)*time  + t_cnt;
}


/*
**********************************************************************
*
*                    OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds. (For profiling only)

   If you have a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling only.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;               /* eliminate effect of prescaler */
  return (1000*Cycles)/(OS_TIMER_FREQ/1000);
}


/*********************************************************************
*
*          Communication (UART)
*
**********************************************************************
*/


#if (OS_UART==1)

  void OS_COM_Init(void) {
    unsigned long ubrlcr_value;
    unsigned char utemp;
    unsigned long ltemp;
    int pll;


    /* Check bit siz of sysflags2. If it is low we're at 74mhz, else 13mhz */
    if (__SYSFLG2 & 0x40) {
      pll = 0;
    }
    else {
      pll = 1;
    }

    utemp   = __PBDDR;
    utemp   &= ~0x02ul;     /* Set bit one of prallel reg b to out */
    __PBDDR = utemp;

    ubrlcr_value = 0;

#if (OS_BAUDRATE==9600)
    ubrlcr_value |= pll ? 23 : 0x0b;
#elif (OS_BAUDRATE==38400)
    ubrlcr_value |= pll ? 5 : 0x02;
#else
  #error Please specify divisor.
#endif
    
    ltemp = (unsigned long) (8 - 5);
    ltemp <<= 17;
    ubrlcr_value |= ltemp;

    ltemp = 0;    /* parity==0 */
    ltemp <<= 13;
    ubrlcr_value |= ltemp;

    __UBRLCR1 = ubrlcr_value;
    __INTMR1 |= 0x2000ul;    /* enable RX */

    ltemp     = __SYSCON1;
    ltemp    |= 0x100; // Bit 8 == enable uart 1 
    __SYSCON1 = ltemp;

    __PBDR = 2;
  }


  void OS_ISR_rx(void) {
    OS_U16 data;
    data = __UARTDR1;     
    if (!(data&0x0300)) {
      OS_OnRx(data & 0xff);
    }
  }
  

  void OS_ISR_tx(void) {
    __INTMR1 &= ~0x1000ul;    /* disable TX */
    OS_OnTx();
  }


  void OS_COM_Send1(OS_U8 c) {
    __INTMR1 |= 0x1000ul;     /* enable TX */
    __UARTDR1 = (OS_U16) c;
  }
  

#else  /* UART handlers not defined, using dummys */

  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }

#endif

/*
**********************************************************************
*
*                    OS_irq_handler
*
**********************************************************************

Purpose:
   Detect reason for IRQ and call correspondig service routine.
*/

void OS_irq_handler(void) {
  OS_U32 status;

  OS_EnterInterrupt();
  OS_USER_irq_func();           /* in userirq.c */
  status = __INTSR1 & __INTMR1;
#if (OS_UART==1)
  if (status & 0x1000) {
    OS_ISR_tx();
  }
  if (status & 0x2000) {
    OS_ISR_rx();
  }
#endif
  if (status & 0x0100) {
    __TC1D    = TIMER_RELOAD;   /* Load counter */      
    __TC1EOI  = 1;              /* Clear interrupt. */
    OS_TickHandler();
  }
  OS_LeaveInterrupt();
}


